#The Object 
import math 

import pandas as pd 
import numpy as np



from matplotlib.ticker import AutoMinorLocator,FormatStrFormatter
import matplotlib.pyplot as plt
from matplotlib import style
from matplotlib.widgets import Cursor,Button


style.use('ggplot')








#    
class GrainStatsDes():

    def __init__(self, s=None):
        self.s=s

    def Infohelp():
        print('''A suite of codes that calculate the Graphic Mean, Graphic Standard  Deviation, Kurtosis and Skewness''')
    
    
    
    def GraphicMean(self, data):
    
        '''A function to calculate the graphic mean for the Data using the 16th, 50th, 84th percentiles'''

        self.data =data
    #     percentiles = [16,50,84]
        phi_percentile=list(self.data['Percentile_Grain_size_(Phi)'])
   
        
        sumation=phi_percentile[1] + phi_percentile[3] + phi_percentile[5]
     
    
        graphicMean= round((sumation/3),2)
        
        return graphicMean

    
    def GraphicStandard_Dev(self,data):

    
        '''
        Method to calculate the graphic standard deviation for the Data 
        using the 95th, 5th, 84th,16th percentiles
        '''
        self.data = data
    #     percentiles_right = [95,5]
    #     percentiles_left = [84,16]
        phi_percentile=list(self.data['Percentile_Grain_size_(Phi)']) 
    
    
    
#     for i in percentiles_left:
#         left_side.append(round(np.percentile(data.Phi_scale,i),2))
    
    
#     for j in percentiles_right:
#         right_side.append(round(np.percentile(data.Phi_scale,j),2))

        left_math = phi_percentile[5] - phi_percentile[1]
        right_math = phi_percentile[6] - phi_percentile[0]
    
        graphic_std = round((left_math/ 4) + (right_math/6.6),2)
        
        return graphic_std
    
    
    def GraphicSkewness(self,data):

        '''
        Method for the the graphic_skewness using the 5th, 16th, 50th, 84th,95th percentiles
        '''
        self.data =data
        #percentiles = [5,16,50,84,95]
    
        phi_percentile=list(self.data['Percentile_Grain_size_(Phi)'])
    
    
    
        left_numerator = phi_percentile[5] + phi_percentile[1]- (2 * phi_percentile[3])
        right_numerator = (phi_percentile[6] + phi_percentile[0])- (2* phi_percentile[3])
    
        left_denometer = 2 *(phi_percentile[5]- phi_percentile[1])
        right_denometer = 2 *(phi_percentile[6] - phi_percentile[0])
        graph_skew = round((left_numerator/left_denometer) + (right_numerator/right_denometer),3)



        return graph_skew
    
    
    
    def GraphicKurtosis(self,data):

        '''
        Method for the Graphic Kurtosis, using the 5th, 25th,75th and the 95th percentile
        '''
        self.data = data
    #     percentile = [5,25,75,95]
    
        phi_percentile=list(self.data['Percentile_Grain_size_(Phi)'])
    
        numerator = phi_percentile[6] - phi_percentile[0]
        denumator = 2.44*(phi_percentile[4] - phi_percentile[3])
    
        graphic_kurtosis = round((numerator / denumator),2)
        return graphic_kurtosis
    
    





__all__ = ['grainpreprocess','grainviz','grainstats','grainstatsdata']





from sedipack.grainpreprocess import GrainPreprocess

from sedipack.grainviz import GrainViz

from sedipack.grainstats import GrainStatsDes

from sedipack.grainstatsdata import GrainDataTables



import math 

import pandas as pd 
import numpy as np

class GrainDataTables():
    
    def __init__(self,z=None):
        self.z= z




    def infohelp(self,x=None):
        self.x=x
        print('''Input the very First Bed Data into the BedData function,
        Then when passing the rest of the bed data
        you include the first data initiated and the data points for other beds. ''')
    
    
    
    def BedData(self, name_of_bed,graphic_mean,graphic_std,graphic_skewness,graphic_kurtosis):
        """
        To save to a bed with it is respective mean, standard dev,skewness,kurtosis.
        the returned data will then be passed into the complete the beds function to complete the beds for an outcrop.
        
        """
        self.name = name_of_bed
        self.graphic_mean=graphic_mean
        self.graphic_standard_dev=graphic_std
        self.graphic_skewness = graphic_skewness
        self.graphic_kurtosis= graphic_kurtosis
        
        bed_stat_data = pd.DataFrame({'Name_of_bed':[self.name],
                      'Graphic Mean':[self.graphic_mean],
                      'Graphic Standard deviation':[self.graphic_standard_dev],
                       'Graphic Skewness':[self.graphic_skewness],
                      'Graphic Kurtosis':[self.graphic_kurtosis]
                     })
    
        return bed_stat_data

    def CompleteBeds(self,bed_stat,name_of_bed,graphic_mean,graphic_std,graphic_skewness,graphic_kurtosis):
        """ 
         To save to a bed with it is respective mean, standard dev,skewness,kurtosis.
        
        """
        self.name = name_of_bed
        self.graphic_mean=graphic_mean
        self.graphic_standard_dev=graphic_std
        self.graphic_skewness = graphic_skewness
        self.graphic_kurtosis= graphic_kurtosis
        
        update={'Name_of_bed':self.name,
                      'Graphic Mean':self.graphic_mean,
                      'Graphic Standard deviation':self.graphic_standard_dev,
                       'Graphic Skewness':self.graphic_skewness,
                      'Graphic Kurtosis':self.graphic_kurtosis
                     }
        self.bed_stat =bed_stat.append(update,ignore_index=True)
    
        return self.bed_stat
    
    def SaveBeds(self,data,name_of_rock_unit):

        '''
        Saves Bed Grains stats from the CompleteBeds and creates a CSV file in your Present working Directory.
        '''
        
        self.data=data
        self.name= name_of_rock_unit
    
        self.data.to_csv(f'{self.name}.csv')
    
    
    



import math 

import pandas as pd 
import numpy as np


class GrainPreprocess():

    '''
    Class to preprocess grains for sedimentological analysis
    Beds and grain conversions
    '''

    def __init__(self,d=None):
        self.d = None

    def MM_ToPhiScale(self,sieveSize):

        """
        Method to convert an array/list of grain seive sizes in mm to phi scale
        
        args:: 
            d: array of grain sieve sizes
        
        Returns the phi reading for sedimentological analysis
        """

        self.d = sieveSize

        assert isinstance(self.d, list), f"seiveSize should be a list, but {type(self.d)} was passed."
        phi=[round(-(math.log2(i)/math.log2(2)),2) for i in self.d]

        return phi
    
    def Bed_To_100(self,bed):

        """
        Method to scaleup beds and truncate beds greater than 100 to 100
        args::
            bed: Bed
        Returns:
            Scaled Bed
        
        """
        self.bed = bed
        assert isinstance(self.bed,list), f"Bed should be a list, but {type(self.phi_bed)} was passed."
        total=sum(self.bed)
        bed =[round(((i/total) *100),2) for i in self.bed]
    
        return bed
    
    
    def CummulateBed(self,bed):

        """
        Method to return the cummulative score of an array (0-100)
        """
        self.bed = bed
        assert isinstance(self.bed, list), f"Bed should be a list, but {type(self.phi_bed)} was passed."
        from itertools import accumulate
   
    
    
        cum_bed=list(accumulate(self.bed))
        cum_beds=[round(i,2) for i in cum_bed]

        return cum_beds




    def Percentiles(self, bed_data):

        """
        This get the percentile for the data passed. 
        
        Returns  a Dictionary of the percentage passing,percentile of the data
        and the bed_data.
        """

        sedi_passing=[5,16,25,50,75,84,95,100]
        self.bed_data = bed_data
        assert isinstance(self.bed, list), f"bed should be a list, but {type(self.bed_data)} was passed."
        bed=self.bed_data
       
        sed_passing=[str(i)+ '%' for i in sedi_passing]
        percentiles= dict(zip(sed_passing,bed))
   
        return percentiles
        






#The Object 
import math 

import pandas as pd 
import numpy as np



from matplotlib.ticker import AutoMinorLocator,FormatStrFormatter
import matplotlib.pyplot as plt
from matplotlib import style
from matplotlib.widgets import Cursor,Button



class GrainViz():
    def __init__(self,f=None):
        self.f=f        
    
    def InfoHelp():

        '''
        Class Help
        '''
        print( '''Generate a DataFrame(Spread Sheet of the data)\n,
        Initial plot of the Cumulative Mass Retain Vs Phi_scale\n,
        to pick the percentiles of interest on the Phi scale.''')
    
    def DataFrame(self,q,phi,bed_data):

        """
        Argss::
            Passing the dictionary, from the percentile function, 
            the phi scale list/array.
            the bed_data.

        Returns:
            DataFrame object
        """
    
        self.q=q
        self.phi = phi

        assert isinstance(self.phi,list), f"phi must be a list, but {type(self.phi)} was passed."
        self.bed_data = bed_data

        Outcrop_Bed_data= dict(zip(self.phi,self.bed_data))
        Outcrop_Bed= pd.DataFrame(Outcrop_Bed_data.items(), columns= ['Phi_scale','Cummulative_Mass_Retained'])
    
        percentile_table=pd.DataFrame(self.q.items(),columns=['Percentiles','Percentile_Passings' ])
        percentile_table.drop(['Percentile_Passings'],1, inplace=True)
    
        Out=percentile_table.join(Outcrop_Bed)
    
    
        return Out
    

    def DataFrame_PhiPercentiles(self,data,phi_percentile):
        """
        Argss:: Data(DataFrame)
        Phi_percentile (list or array of the precentiles)


        Returns:
        DataFrame.
        
        
        """
       
        self.data =data
        self.phi_percentile = phi_percentile
        assert isinstance(self.phi_percentile,list), f"phi_percentile must be a list, but {type(self.phi_percentile)} was passed."
        self.data['Percentile_Grain_size_(Phi)']=self.phi_percentile
        return self.data
    
    



    def DataPlot(self,data,name_of_bed,fig=False,save_data=False):

        '''
        Method to plot data
        args::
            data: Data to be plotted
            name_of_bed: Bed to be plotted
            fig =False (To plot a shart without saving the plot) or Yes (to save the plot)
            save_data = False(This prevent the data from saving as a CSV file to save, pass True)
        '''
        self.data = data
        self.name_of_bed = name_of_bed
        self.fig = fig
        self.save_data = save_data
        
        assert isinstance(self.name_of_bed,str), f"Name of Bed must be a String, but {type(self.name_of_bed)} was passed."

        if self.fig == True or self.save_data == False:
            """Plots the Cuumulative Mass Retained vs the Phi Scale for easy picking of the percentiles"""
            fig,ax=plt.subplots(figsize=(10,8))
            plt.plot(self.data['Phi_scale'],self.data['Cummulative_Mass_Retained'],color='b')
            plt.scatter(self.data['Phi_scale'],self.data['Cummulative_Mass_Retained'], color = 'r')
            ax.xaxis.set_minor_locator(AutoMinorLocator(n=5))
            ax.yaxis.set_minor_locator(AutoMinorLocator(n=5))
            ax.xaxis.set_minor_formatter(FormatStrFormatter("%.1f"))
            ax.yaxis.set_minor_formatter(FormatStrFormatter("%.0f"))
       

            plt.rc('grid', linestyle="-", color='black')
            ax.spines['bottom'].set_color('1.5')
            ax.spines['top'].set_color('1.5')
            ax.spines['right'].set_color('1.5')
            ax.spines['left'].set_color('1.5')
            plt.grid(True, which='minor', color='k', linestyle='-')
            plt.grid(True, which='major', color='r', linestyle='-')
 
            plt.xlabel('Grain Size (Phi)',fontsize=16)
            plt.ylabel('Cumulative Mass Retained (%)',fontsize=16)
            plt.title(f'Cummulative Frequncy Curve {self.name_of_bed}',fontsize=20,fontweight='bold')
            fig.savefig(f'{self.name_of_bed}.png',pdi=fig.dpi)
           
            
        
        elif self.fig == False or self.save_data == True:
            """Plots the Cuumulative Mass Retained vs the Phi Scale for easy picking of the percentiles"""
            fig,ax=plt.subplots(figsize=(10,8))
            plt.plot(self.data['Phi_scale'],self.data['Cummulative_Mass_Retained'],color = 'b')
            plt.scatter(self.data['Phi_scale'],self.data['Cummulative_Mass_Retained'],color = 'r')
            ax.xaxis.set_minor_locator(AutoMinorLocator(n=5))
            ax.yaxis.set_minor_locator(AutoMinorLocator(n=5))
            ax.xaxis.set_minor_formatter(FormatStrFormatter("%.1f"))
            ax.yaxis.set_minor_formatter(FormatStrFormatter("%.0f"))
       

            plt.rc('grid', linestyle="-", color='black')
            ax.spines['bottom'].set_color('1.5')
            ax.spines['top'].set_color('1.5')
            ax.spines['right'].set_color('1.5')
            ax.spines['left'].set_color('1.5')
            plt.grid(True, which='minor', color='k', linestyle='-')
            plt.grid(True, which='major', color='r', linestyle='-')
 
            plt.xlabel('Grain Size (Phi)',fontsize=16)
            plt.ylabel('Cumulative Mass Retained (%)',fontsize=16)
            plt.title(f'Cummulative Frequncy Curve {self.name_of_bed}',fontsize=20,fontweight='bold')
            
            data.to_csv(f'{self.name_of_bed}.csv', index=False)
        
        elif self.fig == True or self.save_data == True:
            """Plots the Cuumulative Mass Retained vs the Phi Scale for easy picking of the percentiles"""
            fig,ax=plt.subplots(figsize=(10,8))
            plt.plot(self.data['Phi_scale'],self.data['Cummulative_Mass_Retained'],color = 'b')
            plt.scatter(self.data['Phi_scale'],self.data['Cummulative_Mass_Retained'],color = 'r')
            ax.xaxis.set_minor_locator(AutoMinorLocator(n=5))
            ax.yaxis.set_minor_locator(AutoMinorLocator(n=5))
            ax.xaxis.set_minor_formatter(FormatStrFormatter("%.1f"))
            ax.yaxis.set_minor_formatter(FormatStrFormatter("%.0f"))
       

            plt.rc('grid', linestyle="-", color='black')
            ax.spines['bottom'].set_color('1.5')
            ax.spines['top'].set_color('1.5')
            ax.spines['right'].set_color('1.5')
            ax.spines['left'].set_color('1.5')
            plt.grid(True, which='minor', color='k', linestyle='-')
            plt.grid(True, which='major', color='r', linestyle='-')
 
            plt.xlabel('Grain Size (Phi)',fontsize=16)
            plt.ylabel('Cumulative Mass Retained (%)',fontsize=16)
            plt.title(f'Cummulative Frequncy Curve {self.name_of_bed}',fontsize=20,fontweight='bold')
            fig.savefig(f'{self.name_of_bed}.png',pdi=fig.dpi)
            data.to_csv(f'{self.name_of_bed}.csv', index=False)
            
            
        else:
              
            """Plots the Cuumulative Mass Retained vs the Phi Scale for easy picking of the percentiles"""
            fig,ax=plt.subplots(figsize=(10,8))
            plt.plot(self.data['Phi_scale'],self.data['Cummulative_Mass_Retained'],color = 'b')
            plt.scatter(self.data['Phi_scale'],self.data['Cummulative_Mass_Retained'],color = 'r')
            ax.xaxis.set_minor_locator(AutoMinorLocator(n=5))
            ax.yaxis.set_minor_locator(AutoMinorLocator(n=5))
            ax.xaxis.set_minor_formatter(FormatStrFormatter("%.1f"))
            ax.yaxis.set_minor_formatter(FormatStrFormatter("%.0f"))
       

            plt.rc('grid', linestyle="-", color='black')
            ax.spines['bottom'].set_color('1.5')
            ax.spines['top'].set_color('1.5')
            ax.spines['right'].set_color('1.5')
            ax.spines['left'].set_color('1.5')
            plt.grid(True, which='minor', color='k', linestyle='-')
            plt.grid(True, which='major', color='r', linestyle='-')
 
            plt.xlabel('Grain Size (Phi)',fontsize=16)
            plt.ylabel('Cumulative Mass Retained (%)',fontsize=16)
            plt.title(f'Cummulative Frequncy Curve {self.name_of_bed}',fontsize=20,fontweight='bold')
           
   
    
    
    
    
